from django.apps import apps 
from django.urls import reverse
from django.conf import settings
from packagebinder.bind import PackageBinding, SettingsBinding, TaskBinding, SidebarBinding
import logging 

logger = logging.getLogger(__name__)
app_config = apps.get_app_config('django_eveonline_timerboard')

def create_bindings():
    sidebar_bindings = apps.get_app_config('packagebinder').sidebar_bindings
    sidebar_bindings['django_eveonline_connector'].add_child_menu_item({
        "fa_icon": "fa-clock",
        "name": "Timerboard",
        "url": reverse("django-eveonline-timerboard-view"),
    })



from django.shortcuts import render, redirect
from django.contrib import messages
from .forms import EveTimerForm
from .models import EveTimer, EveTimerType
from datetime import datetime, timedelta 
from django.utils import timezone
from django.contrib.auth.decorators import login_required, permission_required

@login_required
@permission_required('django_eveonline_timerboard.add_evetimer', raise_exception=True)
def add_timer(request):
    if request.method == "POST":
        form = EveTimerForm(request.POST)

        if form.is_valid():
            EveTimer(
                name=form.cleaned_data['name'],
                timer=datetime.utcnow() + timedelta(days=form.cleaned_data['days'], hours=form.cleaned_data['hours'], minutes=form.cleaned_data['minutes'], seconds=form.cleaned_data['seconds']),
                type=form.cleaned_data.get('type'),
                location=form.cleaned_data['location'],
                user=request.user,
            ).save()
            messages.success(request, 'Succesfully added timer: %s' % form.cleaned_data['name'])
            return redirect('django-eveonline-timerboard-view')
        else:
            messages.error(request, 'Failed to add timer: %s' % str(form.errors.as_text))
            return redirect('django-eveonline-timerboard-view')


@login_required
@permission_required('django_eveonline_timerboard.delete_evetimer', raise_exception=True)
def remove_timer(request, pk):
    EveTimer.objects.get(pk=pk).delete()
    return redirect('django-eveonline-timerboard-view')
      

@login_required
@permission_required('django_eveonline_timerboard.view_evetimer', raise_exception=True)
def view_timerboard(request):
    timers = EveTimer.objects.filter(timer__gte=timezone.now())
    context = {
        'timers': timers,
        'types': EveTimerType.objects.all(),
        'form': EveTimerForm()
    }
    return render(request, 'django_eveonline_timerboard/adminlte/timerboard.html', context)


# Generated by Django 2.2.13 on 2020-11-21 00:18

from django.db import migrations

def load_default_types(apps, schema_editor):
    EveTimerType = apps.get_model('django_eveonline_timerboard', 'EveTimerType')
    types = [
        {
            "name": "Shield",
            "color": "Light Blue",
        },
        {
            "name": "Armor",
            "color": "Yellow",
        },
        {
            "name": "Structure",
            "color": "Red",
        }
    ]
    if EveTimerType.objects.all().count() == 0:
        for timer_type in types:
            EveTimerType.objects.create(name=timer_type['name'], color=timer_type['color'])

def unload_default_types(apps, schema_editor):
    EveTimerType = apps.get_model('django_eveonline_timerboard', 'EveTimerType')
    types = ['Shield', 'Armor', 'Structure']
    EveTimerType.objects.filter(name__in=types).delete()

class Migration(migrations.Migration):

    dependencies = [
        ('django_eveonline_timerboard', '0002_auto_20190917_2024'),
    ]

    operations = [
        migrations.RunPython(load_default_types, reverse_code=unload_default_types)
    ]



from django import forms
from .models import EveTimer, EveTimerType

class EveTimerForm(forms.Form):
    name = forms.CharField(max_length=64, required=True)
    days = forms.IntegerField(min_value=0, required=True)
    hours = forms.IntegerField(min_value=0, max_value=24, required=True)
    minutes = forms.IntegerField(min_value=0, max_value=60, required=True)
    seconds = forms.IntegerField(min_value=0, max_value=60, required=True)
    type = forms.ModelChoiceField(queryset=EveTimerType.objects.all())
    location = forms.CharField(max_length=32, required=True)


from django.db import models
from django_eveonline_connector.models import EveCharacter, EveCorporation, EveAlliance
from django.contrib.auth.models import User, Group



class EveTimer(models.Model):
    name = models.CharField(max_length=64)
    type = models.ForeignKey("EveTimerType", on_delete=models.SET_NULL, null=True, blank=True)
    location = models.CharField(max_length=32, null=True, blank=True)
    timer = models.DateTimeField(auto_now=False, auto_now_add=False)
    user = models.ForeignKey(User, null=True, blank=True, on_delete=models.SET_NULL)

    def __str__(self):
        return self.name

class EveTimerType(models.Model):
    bootstrap_colors = (
        ("primary", "Blue"),
        ("secondary", "Gray"),
        ("success", "Green"),
        ("danger", "Red"),
        ("warning", "Yellow"),
        ("info", "Light Blue"),
        ("light", "White"),
        ("dark", "Black")
    )
    
    name = models.CharField(max_length=32)
    color = models.CharField(max_length=32, choices=bootstrap_colors)

    def __str__(self):
        return self.name


# Generated by Django 2.2.4 on 2019-09-16 18:44

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('auth', '0011_update_proxy_permissions'),
    ]

    operations = [
        migrations.CreateModel(
            name='EveTimer',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=64)),
                ('type', models.CharField(choices=[('OFFENSIVE', 'Offensive'), ('DEFENSIVE', 'Defensive'), ('THIRD_PARTY', 'Third Party')], max_length=32)),
                ('timer', models.DateTimeField()),
                ('groups', models.ManyToManyField(blank=True, null=True, to='auth.Group')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



from django.urls import path
from . import views

urlpatterns = [
    path('timerboard/timer/add/', views.add_timer, name="django-eveonline-timerboard-timer-add"),
    path('timerboard/timer/remove/<int:pk>/', views.remove_timer, name="django-eveonline-timerboard-timer-remove"),
    path('timerboard/view/', views.view_timerboard, name="django-eveonline-timerboard-view"),
]



from django.apps import AppConfig


class DjangoEveonlineTimerboardConfig(AppConfig):
    name = 'django_eveonline_timerboard'
    verbose_name = 'EVE Online Timers'
    url_slug = 'eveonline'

    def ready(self):
        from .bindings import create_bindings
        create_bindings()


# Generated by Django 2.2.4 on 2019-09-17 20:24

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_eveonline_timerboard', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='EveTimerType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=32)),
                ('color', models.CharField(choices=[('primary', 'Blue'), ('secondary', 'Gray'), ('success', 'Green'), ('danger', 'Red'), ('warning', 'Yellow'), ('info', 'Light Blue'), ('light', 'White'), ('dark', 'Black')], max_length=32)),
            ],
        ),
        migrations.RemoveField(
            model_name='evetimer',
            name='groups',
        ),
        migrations.AddField(
            model_name='evetimer',
            name='location',
            field=models.CharField(blank=True, max_length=32, null=True),
        ),
        migrations.AlterField(
            model_name='evetimer',
            name='type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='django_eveonline_timerboard.EveTimerType'),
        ),
    ]
